package com.mcapraro.ld29
{
	import flash.media.SoundTransform;
	import flash.media.SoundChannel;
	import flash.media.Sound;
	import starling.events.TouchPhase;
	import starling.events.TouchEvent;
	import starling.display.Image;
	import starling.display.Sprite;
	import Box2D.Dynamics.Contacts.b2CircleContact;
	import Box2D.Dynamics.Contacts.b2PolyAndCircleContact;
	import citrus.objects.platformer.box2d.Sensor;
	import flash.ui.Keyboard;
	import starling.events.KeyboardEvent;
	import Box2D.Dynamics.b2Body;
	import citrus.core.CitrusObject;
	import citrus.view.starlingview.StarlingArt;
	import starling.textures.Texture;
	import Box2D.Common.Math.b2Vec2;
	import citrus.objects.CitrusSprite;
	import flash.geom.Point;
	import starling.display.Shape;
	import com.greensock.TweenMax;
	import citrus.physics.box2d.Box2DUtils;
	import Box2D.Dynamics.Contacts.b2Contact;
	import citrus.objects.platformer.box2d.Platform;
	import citrus.utils.objectmakers.ObjectMaker2D;
	import citrus.physics.box2d.Box2D;
	import org.osflash.signals.Signal;
	import citrus.core.starling.StarlingState;

	/**
	 * @author Michelangelo Capraro (m&#64;mcapraro.com)
	 */
	public class Level1State extends StarlingState
	{
		public var onRestart:Signal;		

		private var _firstRun:Boolean;
		private var _box2D:Box2D;
		private var _level:Level1;
		private var _hero:VoleHero;
		private var _includes:Array;
		private var _surfaceShape:Shape;
		private var _nut:Nut;
		private var maleableSurface:Vector.<ShiftingPlatform>;
		private var _tunnelFloor:TiledBackground;
		private var _tunnelBackground:TiledBackground;
		private var _skyBackground:TiledBackground;
		private var _surfaceTexture:Texture;
		private var _hillSprite:CitrusSprite;
		private var _branch:Platform;
		private var _winSensor:Sensor;
		private var dialog:Sprite;
		private var walnutReleased:Boolean;
		private var bgMusicChannel:SoundChannel;
		
		public function Level1State(firstRun:Boolean = false)
		{
			trace("Level1State created");
			super();
			
			_includes = [];
			
			_level = new Level1();
			
			_firstRun = firstRun;
			
			onRestart = new Signal();
		}

		override public function initialize():void
		{
			super.initialize();
			
			_box2D = new Box2D("box2D");
			//_box2D.visible = true;
			add(_box2D);
						
			setupMap();
			setupHero();
			setupNut();
			setupSensors();
			setupCoins();
			setupUI();
			setupCamera();
						
			start();
		}

		private function setupUI():void
		{
			var restartButton:Sprite = new Sprite();
			restartButton.addChild(new Image(Assets.assets.getTexture("restart-button")));
			restartButton.x = stage.stageWidth - restartButton.width - 10;
			restartButton.y = stage.stageHeight - restartButton.height - 10;
			restartButton.addEventListener(TouchEvent.TOUCH, onRestartButton);
			addChild(restartButton);


			var goButton:Sprite = new Sprite();
			goButton.addChild(new Image(Assets.assets.getTexture("go-button")));
			goButton.x = (stage.stageWidth - goButton.width) / 2;
			goButton.y = stage.stageHeight - goButton.height - 10;
			goButton.addEventListener(TouchEvent.TOUCH, onGoButton);
			addChild(goButton);
		}

		private function onGoButton(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) releaseWalnut();
		}

		private function onRestartButton(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) onRestart.dispatch();
		}

		private function setupCoins():void
		{
			var leaves:Vector.<CitrusObject> = getObjectsByType(Leaf);
			for (var i:int = 0; i < leaves.length; i++) {
				var leaf:Leaf = leaves[i] as Leaf;
				leaf.group = 4;
				leaf.collectorClass = Nut;
				leaf.onBeginContact.add(onLeafCollected);
				
			}
		}

		private function onLeafCollected(callback:b2CircleContact):void
		{
			trace("collected! " + callback);
		}

		private function setupSensors():void
		{
			var failSensor:Sensor = getObjectByName("failSensor") as Sensor;
			failSensor.onBeginContact.add(onFailSensor);

			_winSensor = getObjectByName("winSensor") as Sensor;
			_winSensor.onBeginContact.add(onWinSensor);
		}

		private function onWinSensor(callback:b2Contact):void
		{

			if (Box2DUtils.CollisionGetOther(_winSensor, callback) == _hero) return;

			_winSensor.onBeginContact.remove(onWinSensor);
			
			_nut.updateCallEnabled = false;
			_nut.stopSound();
			
			trace("win! " + callback);
			
			//TODO: need to do somehting fun here
			camera.target = null;
			
			_hero.controlsEnabled = false;
			
			TweenMax.delayedCall(1, function():void {
				camera.easing.x = 0.05;
				camera.target = _hero;
			});
			
			showWinScreen();
			
		}

		private function onFailSensor(callback:b2PolyAndCircleContact):void
		{
			trace("lose! " + callback);
			camera.target = null;
			TweenMax.delayedCall(2, function():void {
				camera.easing.x = 0.02;
				camera.target = _hero;
			});
			
			_nut.updateCallEnabled = false;
			_nut.stopSound();
			
			showFailScreen();
		}

		private function showFailScreen():void
		{
			_hero.controlsEnabled = false;
			
			dialog = new Sprite();

			var dimmer:Shape = new Shape();
			dimmer.graphics.beginFill(0x000000, .6);
			dimmer.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			dimmer.graphics.endFill();
			dialog.addChild(dimmer);
			
			var title:Image = new Image(Assets.assets.getTexture("fail_screen"));
			dialog.addChild(title);
			
			title.x = (stage.stageWidth - title.width) / 2;
			title.y = (stage.stageHeight - title.height) / 2;
			dialog.alpha = 0;
			dialog.addEventListener(TouchEvent.TOUCH, onDismissFailScreen);
			addChild(dialog);
			
			addEventListener(KeyboardEvent.KEY_DOWN, onKeyFailScreen);	
			
			TweenMax.to(dialog, .5, {delay:3, alpha:1});
			
		}

		private function onKeyFailScreen(event:KeyboardEvent):void
		{
			onRestart.dispatch();
		}

		private function onDismissFailScreen(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) onRestart.dispatch();
		}

		private function showWinScreen():void
		{
			_hero.controlsEnabled = false;
			
			dialog = new Sprite();

			var dimmer:Shape = new Shape();
			dimmer.graphics.beginFill(0x000000, .6);
			dimmer.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			dimmer.graphics.endFill();
			dialog.addChild(dimmer);
			
			var title:Image = new Image(Assets.assets.getTexture("win_screen"));
			dialog.addChild(title);
			
			title.x = (stage.stageWidth - title.width) / 2;
			title.y = (stage.stageHeight - title.height) / 2;
			dialog.alpha = 0;
			dialog.addEventListener(TouchEvent.TOUCH, onDismissWinScreen);
			addChild(dialog);
			
			addEventListener(KeyboardEvent.KEY_DOWN, onKeyWinScreen);	
			
			TweenMax.to(dialog, .5, {delay:1, alpha:1});
			
		}

		private function onKeyWinScreen(event:KeyboardEvent):void
		{
			onRestart.dispatch();
		}

		private function onDismissWinScreen(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) onRestart.dispatch();
		}


		private function releaseWalnut():void
		{
			if (walnutReleased) return;
			walnutReleased = true;
			_branch.body.SetType(b2Body.b2_dynamicBody);
			_nut.body.ApplyImpulse(new b2Vec2(-150,-150), new b2Vec2(_nut.body.GetPosition().x - 1, _nut.body.GetPosition().y - 1));
			camera.target = _nut;
		}

		private function setupNut():void
		{
			_nut = getObjectByName("nut") as Nut;
			_nut.group = 3;
		}


		private function setupHero():void
		{
			_hero = getObjectByName("hero") as VoleHero;
			_hero.group = 4;
		}

		private function setupCamera():void
		{
			//camera.bounds = new Rectangle(0,0,2000,2000);
			camera.target = _hero;
			camera.enabled = true;
			camera.easing.x = 0.05;
			camera.easing.y = 0;
			camera.center = new Point(.5, .7);
			
			camera.allowZoom = true;
			camera.setZoom(.8);
			camera.reset();
		}

		
		private function setupMap():void
		{
			// create the physics bodies from the SWF
			ObjectMaker2D.FromMovieClip(_level);						
			
			var surface:Platform = getObjectByName("surface") as Platform;
			var surfaceWidth:Number = surface.width;
			var sliceWidth:Number = 30;
			
			// create a ground texture.t this is a hack but we have to do it this way for now
			var tempTextureObj:CitrusSprite = new CitrusSprite("tempTexture", {view:Assets.assets.getTexture("surface-tile"), width:512, height:32});
			add(tempTextureObj);
			
			_surfaceTexture = Texture.fromBitmapData(Utils.copyAsBitmapData((view.getArt(tempTextureObj) as StarlingArt)));
			remove(tempTextureObj);
			
			// this is the ground level until we get time to make this work better
			_surfaceShape = new Shape();
			_surfaceShape.graphics.beginTextureFill(_surfaceTexture);
			//_surfaceShape.graphics.beginFill(0xff0000, 1);
			_surfaceShape.graphics.drawRect(surface.x - surface.width / 2, surface.y - surface.height / 2, surface.width, surface.height);
			_surfaceShape.graphics.endFill();
			var surfaceObj:CitrusSprite = new CitrusSprite("surfaceObj", {view:_surfaceShape, group:2, touchable:false });
			add(surfaceObj);

			// create the individual platform elements for the maleable ground level
			var sliceSpacing:int = 20;
			var segmentCount:int = Math.ceil(surfaceWidth / sliceSpacing);
			var segment:ShiftingPlatform;
			var lastSegment:ShiftingPlatform;
			
			maleableSurface = new Vector.<ShiftingPlatform>();
			
			for (var i:int = 0; i < segmentCount; i++) {
				lastSegment = segment;
				segment = new ShiftingPlatform("segment_" + i, {width:sliceWidth, height:surface.height});
				segment.x = surface.x + (i * sliceSpacing) - (surface.width / 2);
				segment.y = surface.y;
				segment.onBeginContact.add(onBeginShiftGroundPlatform);

				add(segment);
				
				maleableSurface.push(segment);
			}
			
			
			var bgWidth:int = 8000;
			
			// create the tiling elements 
			var solidSkyShape:Shape = new Shape();
			solidSkyShape.graphics.beginFill(0x00ffff, 1);
			solidSkyShape.graphics.drawRect(0, 0, bgWidth, 120);
			solidSkyShape.graphics.endFill();
			var solidSkyBackground:CitrusSprite = new CitrusSprite("solidSkyBackground", {view:solidSkyShape, x:0, y:0, offsetX:-bgWidth/2, group:0, parallaxX:0, parallaxY:0});
			add(solidSkyBackground);

			_skyBackground = new TiledBackground("_skyBackground", {x:0, y:-30, offsetX: -bgWidth/2, width:bgWidth, height:512, group:0, texture:"sky-tile", parallaxX:.6, parallaxY:0, overlapX:1});
			add(_skyBackground);

			_tunnelBackground = new TiledBackground("_tunnelBackground", {x:0, y:479, offsetX: -bgWidth/2, width:bgWidth, height:128, group:0, texture:"tunnel-tile", parallaxX:.6, parallaxY:0, overlapX:1});
			add(_tunnelBackground);

			_tunnelFloor = new TiledBackground("_tunnelFloor", {x:0, y:_tunnelBackground.y + _tunnelBackground.height - 10, offsetX: -bgWidth/2, width:bgWidth, height:32, group:0, texture:"floor-tile", parallaxX:.8, parallaxY:0, overlapX:1});
			add(_tunnelFloor);
			
			// get any parts in the swf we need to rebuild
			var hillShape:Shape = new Shape();
			ShapeConverter.convertToGeometry(_level.getChildByName("hill"), hillShape);
			_hillSprite = new CitrusSprite("hillShape", { x:_level.getChildByName("hill").x, y:_level.getChildByName("hill").y, group:0, view:hillShape, touchable:false} );
			add(_hillSprite);

			var rock:Platform = getObjectByName("rock") as Platform;
			rock.view = Assets.assets.getTexture("rock");
			rock.group = 2;

			rock = getObjectByName("rock2") as Platform;
			rock.view = Assets.assets.getTexture("rock");
			rock.group = 2;

			_branch = getObjectByName("branch") as Platform;
			_branch.view = Assets.assets.getTexture("branch");
			_branch.group = 4;

			// this is the other side of the gap
			var solidSurface:Platform = getObjectByName("solidSurface") as Platform;
			var solidSurfaceShape:Shape = new Shape();
			solidSurfaceShape.graphics.beginTextureFill(_surfaceTexture);
			solidSurfaceShape.graphics.drawRect(0, 0, solidSurface.width, solidSurface.height);
			solidSurfaceShape.graphics.endFill();
			solidSurface.view = solidSurfaceShape;
			solidSurface.group = 4;
			solidSurface.offsetY = -8;
			
			// the opening to the tunnel
			var openingSprite:CitrusSprite = new CitrusSprite("openingSprite", {view:Assets.assets.getTexture("opening"), x:solidSurface.x - solidSurface.width / 2 - 118 - 40, y:_tunnelBackground.y - 30, width:182, height:118});
			add(openingSprite);
			
			
			
			remove(surface);
			
			
		}

		private function onBeginShiftGroundPlatform(callback:b2Contact):void
		{
			var collider:VoleHero = Box2DUtils.CollisionGetObjectByType(VoleHero, callback) as VoleHero;
			var groundSlice:ShiftingPlatform = Box2DUtils.CollisionGetObjectByType(ShiftingPlatform, callback) as ShiftingPlatform;
			if (collider == null) return;
									
			shiftGroundAtSegment(groundSlice);
			
		}

		private function shiftGroundAtSegment(segment:ShiftingPlatform):void
		{
			//trace("shiftGroundAtSegment: " + maleableSurface.indexOf(segment));
			
			var segmentIndex:int = maleableSurface.indexOf(segment);
						
			var seg1:ShiftingPlatform = maleableSurface[Math.max(segmentIndex - 2, 0)];
			var seg2:ShiftingPlatform = maleableSurface[Math.max(segmentIndex - 1, 0)];
			var seg3:ShiftingPlatform = maleableSurface[Math.min(segmentIndex + 1, maleableSurface.length - 1)];
			var seg4:ShiftingPlatform = maleableSurface[Math.min(segmentIndex + 2, maleableSurface.length - 1)];
			
			var p1:Point = new Point(seg1.x, seg1.y);
			var p4:Point = new Point(seg4.x, seg4.y);
			var segmentPoint:Point = new Point(segment.x, segment.y - 15);
			
			var p2:Point = Point.interpolate(p1, segmentPoint, .5);
			var p3:Point = Point.interpolate(segmentPoint, p4, .5);
			
			TweenMax.delayedCall(.01, function():void {
				
				_nut.body.SetAwake(true);
				if ((_hero.x > _nut.x && _hero.x < _nut.x + _nut.width) || (_nut.x > _hero.x && _nut.x < _hero.x + _hero.width)) {
					_nut.body.ApplyImpulse(new b2Vec2(0, -7), _hero.body.GetPosition());
				}
				
				
				seg1.y = p1.y;
				seg2.y = p2.y;
				segment.y = segmentPoint.y;
				seg3.y = p3.y;
				seg4.y = p4.y;
							
				_surfaceShape.graphics.clear();
				//_surfaceShape.graphics.beginFill(0xff0000, 1);
				_surfaceShape.graphics.beginTextureFill(_surfaceTexture);
				
				_surfaceShape.graphics.moveTo(maleableSurface[0].x, maleableSurface[0].y);
				for (var i:int = 0; i < maleableSurface.length; i++) {
					_surfaceShape.graphics.lineTo(maleableSurface[i].x, maleableSurface[i].y - (segment.height / 2));
				}
				_surfaceShape.graphics.lineTo(maleableSurface[maleableSurface.length - 1].x + (maleableSurface[maleableSurface.length - 1].width / 2) , maleableSurface[maleableSurface.length - 1].y - (segment.height / 2));
				_surfaceShape.graphics.lineTo(maleableSurface[maleableSurface.length - 1].x + (maleableSurface[maleableSurface.length - 1].width / 2) , maleableSurface[maleableSurface.length - 1].y - (segment.height / 2) + segment.height);
				for (i = maleableSurface.length - 1; i > -1; i--) {
					_surfaceShape.graphics.lineTo(maleableSurface[i].x, maleableSurface[i].y - (segment.height / 2) + segment.height);
				}
				_surfaceShape.graphics.lineTo(maleableSurface[0].x, maleableSurface[0].y - (segment.height / 2) + segment.height);
				_surfaceShape.graphics.endFill();

				_surfaceShape.graphics.beginFill(0x26211e, 1);
				_surfaceShape.graphics.moveTo(maleableSurface[0].x, maleableSurface[0].y + segment.height);
				for (i = 0; i < maleableSurface.length; i++) {
					_surfaceShape.graphics.lineTo(maleableSurface[i].x, maleableSurface[i].y - (segment.height / 2) + segment.height);
				}
				_surfaceShape.graphics.lineTo(maleableSurface[maleableSurface.length - 1].x + (maleableSurface[maleableSurface.length - 1].width / 2) , maleableSurface[maleableSurface.length - 1].y - (segment.height / 2) + segment.height);
				_surfaceShape.graphics.lineTo(maleableSurface[maleableSurface.length - 1].x + (maleableSurface[maleableSurface.length - 1].width / 2) , _tunnelBackground.y - 28);
				_surfaceShape.graphics.lineTo(maleableSurface[0].x, _tunnelBackground.y - 28);
				_surfaceShape.graphics.endFill();
				

			});
			
		}

		
		private function start():void
		{
			//TweenMax.delayedCall(4, function():void{_branch.body.SetType(b2Body.b2_dynamicBody);camera.target = _nut;});

			if (_firstRun) {
				//TODO: make this a class, make it more memroy efficient, etc
				
				_hero.controlsEnabled = false;
				
				dialog = new Sprite();

				var dimmer:Shape = new Shape();
				dimmer.graphics.beginFill(0x000000, .6);
				dimmer.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
				dimmer.graphics.endFill();
				dialog.addChild(dimmer);
				
				var title:Image = new Image(Assets.assets.getTexture("start_screen"));
				dialog.addChild(title);
				
				title.x = (stage.stageWidth - title.width) / 2;
				title.y = (stage.stageHeight - title.height) / 2;
				dialog.alpha = 0;
				dialog.addEventListener(TouchEvent.TOUCH, onDismissStartScreen);
				addChild(dialog);
				
				addEventListener(KeyboardEvent.KEY_DOWN, onKeyStartScreen);	
				
				TweenMax.to(dialog, .5, {alpha:1});
			} else {
				onStartScreenFaded();
			}
			
			var bgMusic:Sound = new Assets.background_music();
			bgMusicChannel = bgMusic.play(0, 99999);
			var transform:SoundTransform = new SoundTransform(0.5, 1.0);
			bgMusicChannel.soundTransform = transform;

		}

		private function onKeyStartScreen(event:KeyboardEvent):void
		{
			fadeOutStartScreen();
		}

		private function onDismissStartScreen(event:TouchEvent):void
		{
			if (event.touches[0].phase == TouchPhase.ENDED) fadeOutStartScreen();
		}
		
		private function fadeOutStartScreen():void
		{
			removeEventListener(KeyboardEvent.KEY_DOWN, onKeyStartScreen);
			dialog.removeEventListener(TouchEvent.TOUCH, onDismissStartScreen);
			TweenMax.to(dialog, .5, {alpha:0, onComplete:onStartScreenFaded});
		}
		
		private function onStartScreenFaded():void
		{
			if (dialog != null) removeChild(dialog);
			dialog = null;
			_hero.controlsEnabled = true;
			_nut.updateCallEnabled = true;
			
		}
		
		
		override public function update(timeDelta:Number):void
		{
			super.update(timeDelta);
		}


		override public function destroy():void
		{
			_nut.updateCallEnabled = false;
			_nut.stopSound();
			bgMusicChannel.stop();
			bgMusicChannel = null;
			super.destroy();
		}
	}
}
