package com.mcapraro.ld29
{
	import flash.display.Graphics;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.display.DisplayObject;
	import flash.display.GraphicsEndFill;
	import flash.display.GraphicsPathCommand;
	import flash.display.GraphicsPath;
	import flash.display.GraphicsSolidFill;
	import flash.display.IGraphicsData;
	import starling.display.Shape;
	import flash.display.Shape;
	/**
	 * @author Michelangelo Capraro (m&#64;mcapraro.com)
	 */
	public class ShapeConverter
	{
		public function ShapeConverter() 
		{
			
		}
		
		public static function convertToGeometry(flashShape:flash.display.DisplayObject, starlingShape:starling.display.Shape = null):starling.display.Shape
		{
			if (starlingShape == null) starlingShape = new starling.display.Shape();
			var shapeData:Vector.<IGraphicsData>;
			
			if (flashShape is Sprite || flashShape is MovieClip || flashShape is flash.display.Shape) {
				shapeData = (flashShape["graphics"] as Graphics).readGraphicsData(true);
				
			} else {
				return starlingShape;
			}
			
			var drawNumbers:Vector.<Number>;
						
			for (var i:int; i < shapeData.length; i++) {
				
				//trace("ShapeConverter: convertToGeometry: shapeData [" + shapeData[i] + "]");
				
				// determine the fill color
				if (shapeData[i] is GraphicsSolidFill) {
					//trace("ShapeConverter: convertToGeometry: solidFill");
					starlingShape.graphics.beginFill((shapeData[i] as GraphicsSolidFill).color, (shapeData[i] as GraphicsSolidFill).alpha);
				}
				
				// parse the path and determine the type of commands
				if (shapeData[i] is GraphicsPath) {
					
					var commands:Vector.<int> = (shapeData[i] as GraphicsPath).commands;
					var commandLength:int = commands.length;
					var data:Vector.<Number> = (shapeData[i] as GraphicsPath).data;
					var dataOffset:int = 0;
										
					for (var j:int = 0; j < commandLength; j++) {
						switch(commands[j]) {
							case GraphicsPathCommand.CUBIC_CURVE_TO:
								//trace("\tCUBIC_CURVE_TO");
								
								drawNumbers = data.slice(dataOffset, dataOffset + 6);
								dataOffset = dataOffset + 6;
								
								starlingShape.graphics.cubicCurveTo(drawNumbers[0], drawNumbers[1], drawNumbers[2], drawNumbers[3], drawNumbers[4], drawNumbers[5]);
								break;
								
							case GraphicsPathCommand.CURVE_TO:
								//trace("\tCURVE_TO");

								drawNumbers = data.slice(dataOffset, dataOffset + 4);
								dataOffset = dataOffset + 4;
																
								starlingShape.graphics.curveTo(drawNumbers[0], drawNumbers[1], drawNumbers[2], drawNumbers[3]);
								break;
								
							case GraphicsPathCommand.LINE_TO:
								//trace("\tLINE_TO");

								drawNumbers = data.slice(dataOffset, dataOffset + 2);
								dataOffset = dataOffset + 2;
								
								starlingShape.graphics.lineTo(drawNumbers[0], drawNumbers[1]);
								break;
								
							case GraphicsPathCommand.MOVE_TO:
								//trace("\tMOVE_TO");

								drawNumbers = data.slice(dataOffset, dataOffset + 2);
								dataOffset = dataOffset + 2;
								
								starlingShape.graphics.moveTo(drawNumbers[0], drawNumbers[1]);
								break;
								
							case GraphicsPathCommand.NO_OP:
								//trace("\tNO_OP");
								break;
								
							case GraphicsPathCommand.WIDE_LINE_TO:
								//trace("\tWIDE_LINE_TO");

								drawNumbers = data.slice(dataOffset, dataOffset + 2);
								dataOffset = dataOffset + 2;
								
								starlingShape.graphics.lineTo(drawNumbers[0], drawNumbers[1]);
								break;
								
							case GraphicsPathCommand.WIDE_MOVE_TO:
								//trace("\tWIDE_MOVE_TO");

								drawNumbers = data.slice(dataOffset, dataOffset + 2);
								dataOffset = dataOffset + 2;
								
								starlingShape.graphics.moveTo(drawNumbers[0], drawNumbers[1]);
								break;
						}
						
					}
				}

				// determine the fill color
				if (shapeData[i] is GraphicsEndFill) {
					//trace("ShapeConverter: convertToGeometry: endFill");
					starlingShape.graphics.endFill();
				}
				
				
			}
			
			return starlingShape;
		}
	}
}
